% script for reading AutoCAD polyline data from txt files and calculating all
% necessary variables for T12 vertebra endplate shape analysis.

% load required package
pkg load io

% read data from Aalborg Collection
files = dir("./DATA/AALB*.txt");
for f = 1:length(files)
  % get numeric sample ID
  sID = str2num(files(f).name([6:9]));
  % get endplate side (Upper = 1; Lower = 2)
  if strcmp(files(f).name(11), "U")
    endPS = 1;
  else
    endPS = 2;
  endif
  % get polyline side (interior = 1; exterior = 2)
  if strcmp(files(f).name(13), "i")
    polyS = 1;
  else
    polyS = 2;
  endif
  % read text file, remove NaNs, and keep x and y coordinates of each point
  a = textread(fullfile(files(f).folder, files(f).name));
  b = a(isfinite(a));
  len = length(b);
  % construct two vectors for polygeom function
  for i = 1:(len-2)/3
    x(i) = b(i*3);
    y(i) = b((i*3)+1);
  endfor
  % calculate area and perimeter
  [geom, iner, cpmo] = polygeom(x, y);
  % save the sample ID, area, and perimeter
  poly(f,:) = [sID, endPS, polyS, geom(1) geom(4)];
  % clear variables
  clear a b len x y geom iner cpmo sID endPS polyS
endfor
clear f i files

% read inventory for Aalborg Collection
iAALB = csvread("inventory_AALB.csv");
iAALB(1,:) = [];
% for every listed individual find corresponding polylines and calculate
% variables for upper and lower endplates
snum = 1;
d_DK = {"sample ID", "sex", "min age", "max age", "UiArea", "UiPer", "UeArea", "UePer",...
        "UieArea", "UieAreaRatio", "UiArPerIndex", "UeArPerIndex", "UieArPerIndex",...
        "LiArea", "LiPer", "LeArea", "LePer", "LieArea", "LieAreaRatio", "LiArPerIndex",...
        "LeArPerIndex", "LieArPerIndex"};
for s = 1:size(iAALB,1)
  sID = iAALB(s,1);
  sample_var = poly(poly(:,1)==sID,:);
  sample_name = cstrcat("AALB_", char(num2str(sID,'%04d')));
  sample_sex = iAALB(s,2);
  sample_min_age = iAALB(s,3);
  sample_max_age = iAALB(s,4);
  % both endplates
  if size(sample_var,1) == 4
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  % only upper endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 1
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN};
  % only lower endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 2
    snum += 1;
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age,...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN,...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  endif
  clear Ui Ue Li Le sID sample_var sample_name sample_sex sample_min_age sample_max_age
endfor
%cell2csv("shapeData-AALB.csv", dAALB);
clear poly iAALB



% read data from HolmensKirke Collection
files = dir("./DATA/HOLM*.txt");
for f = 1:length(files)
  % get numeric sample ID
  sID = str2num(files(f).name([6:9]));
  % get endplate side (Upper = 1; Lower = 2)
  if strcmp(files(f).name(11), "U")
    endPS = 1;
  else
    endPS = 2;
  endif
  % get polyline side (interior = 1; exterior = 2)
  if strcmp(files(f).name(13), "i")
    polyS = 1;
  else
    polyS = 2;
  endif
  % read text file, remove NaNs, and keep x and y coordinates of each point
  a = textread(fullfile(files(f).folder, files(f).name));
  b = a(isfinite(a));
  len = length(b);
  % construct two vectors for polygeom function
  for i = 1:(len-2)/3
    x(i) = b(i*3);
    y(i) = b((i*3)+1);
  endfor
  % calculate area and perimeter
  [geom, iner, cpmo] = polygeom(x, y);
  % save the sample ID, area, and perimeter
  poly(f,:) = [sID, endPS, polyS, geom(1) geom(4)];
  % clear variables
  clear a b len x y geom iner cpmo sID endPS polyS
endfor
clear f i files

% read inventory for HolmensKirke Collection
iHOLM = csvread("inventory_HOLM.csv");
iHOLM(1,:) = [];
% for every listed individual find corresponding polylines and calculate
% variables for upper and lower endplates
% append them to the previous "poly" cell array
for s = 1:size(iHOLM,1)
  sID = iHOLM(s,1);
  sample_var = poly(poly(:,1)==sID,:);
  sample_name = cstrcat("HOLM_", char(num2str(sID,'%04d')));
  sample_sex = iHOLM(s,2);
  sample_min_age = iHOLM(s,3);
  sample_max_age = iHOLM(s,4);
  % both endplates
  if size(sample_var,1) == 4
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  % only upper endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 1
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN};
  % only lower endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 2
    snum += 1;
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_DK(snum,:) = {sample_name, sample_sex, sample_min_age, sample_max_age,...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN,...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  endif
  clear Ui Ue Li Le sID sample_var sample_name sample_sex sample_min_age sample_max_age
endfor
cell2csv("shapeData-DK.csv", d_DK);
clear poly iHOLM snum 





% read data from Athens Collection
files = dir("./DATA/ATHE*.txt");
for f = 1:length(files)
  % get numeric sample ID
  sID = str2num(files(f).name([6:9]));
  % get endplate side (Upper = 1; Lower = 2)
  if strcmp(files(f).name(11), "U")
    endPS = 1;
  else
    endPS = 2;
  endif
  % get polyline side (interior = 1; exterior = 2)
  if strcmp(files(f).name(13), "i")
    polyS = 1;
  else
    polyS = 2;
  endif
  % read text file, remove NaNs, and keep x and y coordinates of each point
  a = textread(fullfile(files(f).folder, files(f).name));
  b = a(isfinite(a));
  len = length(b);
  % construct two vectors for polygeom function
  for i = 1:(len-2)/3
    x(i) = b(i*3);
    y(i) = b((i*3)+1);
  endfor
  % calculate area and perimeter
  [geom, iner, cpmo] = polygeom(x, y);
  % save the sample ID, area, and perimeter
  poly(f,:) = [sID, endPS, polyS, geom(1) geom(4)];
  % clear variables
  clear a b len x y geom iner cpmo sID endPS polyS
endfor
clear f i files

% read inventory for Athens Collection
iATHE = csvread("inventory_ATHE.csv");
iATHE(1,:) = [];
% for every listed individual find corresponding polylines and calculate
% variables for upper and lower endplates
snum = 1;
d_GR = {"sample ID", "sex", "age", "UiArea", "UiPer", "UeArea", "UePer",...
        "UieArea", "UieAreaRatio", "UiArPerIndex", "UeArPerIndex", "UieArPerIndex",...
        "LiArea", "LiPer", "LeArea", "LePer", "LieArea", "LieAreaRatio", "LiArPerIndex",...
        "LeArPerIndex", "LieArPerIndex"};
for s = 1:size(iATHE,1)
  sID = iATHE(s,1);
  sample_var = poly(poly(:,1)==sID,:);
  sample_name = cstrcat("ATHE_", char(num2str(sID,'%04d')));
  sample_sex = iATHE(s,2);
  sample_age = iATHE(s,3);
  % both endplates
  if size(sample_var,1) == 4
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_GR(snum,:) = {sample_name, sample_sex, sample_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  % only upper endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 1
    snum += 1;
    Ui = sample_var(sample_var(:,2)==1 & sample_var(:,3)==1, [4:5]);
    Ue = sample_var(sample_var(:,2)==1 & sample_var(:,3)==2, [4:5]);
    d_GR(snum,:) = {sample_name, sample_sex, sample_age, Ui(1), Ui(2),...
                    Ue(1), Ue(2), Ue(1)-Ui(1), (Ue(1)-Ui(1))/Ue(1), (Ui(1)*4*pi)/(Ui(2)^2),...
                    (Ue(1)*4*pi)/(Ue(2)^2), ((Ue(1) - Ui(1))*4*pi)/(Ue(2)^2),...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN};
  % only lower endplate
  elseif size(sample_var,1) == 2 && sample_var(1,2) == 2
    snum += 1;
    Li = sample_var(sample_var(:,2)==2 & sample_var(:,3)==1, [4:5]);
    Le = sample_var(sample_var(:,2)==2 & sample_var(:,3)==2, [4:5]);
    d_GR(snum,:) = {sample_name, sample_sex, sample_age,...
                    NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN, NaN,...
                    Li(1), Li(2), Le(1), Le(2), Le(1)-Li(1), (Le(1)-Li(1))/Le(1),...
                    (Li(1)*4*pi)/(Li(2)^2), (Le(1)*4*pi)/(Le(2)^2), ((Le(1)-Li(1))*4*pi)/(Le(2)^2)};
  endif
  clear Ui Ue Li Le sID sample_var sample_name sample_sex sample_age
endfor
cell2csv("shapeData-GR.csv", d_GR);
clear poly iATHE snum s