% script for checking morphometric variables for sexual dimorphism in both
% population groups. Normality is checked before applying ANOVA or Kruskal
% Wallis and effect size is estimated with omega squared estimator. Results
% are seggregated and saved in a file.

% load necessary package
pkg load io
pkg load statistics

% load data for Greek population
Mvar = csv2cell("shapeData-GR.csv")(1,[4:end]);
data = csvread("shapeData-GR.csv")([2:end],[2,4:end]);

% group data by sex
sex = data(:,1);
Mdata = data(sex == 1,[2:19]);
Fdata = data(sex == 2,[2:19]);

% set threshold for statistical significance
alpha = 0.05;

% prepare results table
T = {"Morphometric Variables", "GR Males normality", "GR Females normality",...
     "F / Chi-sq", "p-value", "ω^2 / ε^2", "DK Males normality",...
     "DK Females normality", "F / Chi-sq", "p-value", "ω^2 / ε^2"};

% check each variable for sexual dimorphism
for i=1:18
  % remove NaN samples
  var_M = Mdata(isfinite(Mdata(:,i)),i);
  var_F = Fdata(isfinite(Fdata(:,i)),i);
  % check for normality
  nor_M = shapiro_wilk_test(var_M);
  nor_F = shapiro_wilk_test(var_F);
  % arrange data and groups
  x = [var_M; var_F];
  group = [ones(length(var_M),1); 2*ones(length(var_F),1)];
  if nor_M > alpha && nor_F > alpha
    [PVAL, ATAB] = anova1(x, group, "off");
    % use ω^2 for estimating effect size in ANOVA
    omega2 = (ATAB{2,2}-ATAB{2,3}*ATAB{3,4})/(ATAB{3,4}+ATAB{4,2});
    T(i+1,[1:6]) = {Mvar{i}, nor_M, nor_F, ATAB{2,5}, PVAL, omega2};
  else
    [PVAL, TBL] = kruskalwallis(x, group, "off");
    % use ε^2 for estimating effect size in Kruskal-Wallis
    epsilon2 = (TBL{2,2}-TBL{2,3}*TBL{3,4})/TBL{4,2};
    T(i+1,[1:6]) = {Mvar{i}, nor_M, nor_F, TBL{2,5}, PVAL, epsilon2};
  endif
endfor

% load data for Danish population
Mvar = csv2cell("shapeData-DK.csv")(1,[5:end]);
data = csvread("shapeData-DK.csv")([2:end],[2,5:end]);

% group data by sex
sex = data(:,1);
Mdata = data(sex == 1,[2:19]);
Fdata = data(sex == 2,[2:19]);

% check each variable for sexual dimorphism
for i=1:18
  % remove NaN samples
  var_M = Mdata(isfinite(Mdata(:,i)),i);
  var_F = Fdata(isfinite(Fdata(:,i)),i);
  % check for normality
  nor_M = shapiro_wilk_test(var_M);
  nor_F = shapiro_wilk_test(var_F);
  % arrange data and groups
  x = [var_M; var_F];
  group = [ones(length(var_M),1); 2*ones(length(var_F),1)];
  if nor_M > alpha && nor_F > alpha
    [PVAL, ATAB] = anova1(x, group, "off");
    % use ω^2 for estimating effect size in ANOVA
    omega2 = (ATAB{2,2}-ATAB{2,3}*ATAB{3,4})/(ATAB{3,4}+ATAB{4,2});
    T(i+1,[7:11]) = {nor_M, nor_F, ATAB{2,5}, PVAL, omega2};
  else
    [PVAL, TBL] = kruskalwallis(x, group, "off");
    % use ε^2 for estimating effect size in Kruskal-Wallis
    epsilon2 = (TBL{2,2}-TBL{2,3}*TBL{3,4})/TBL{4,2};
    T(i+1,[7:11]) = {nor_M, nor_F, TBL{2,5}, PVAL, epsilon2};
  endif
endfor

cell2csv("Sex_Dimorphism.csv", T);